# Generated by Django 3.2.4 on 2021-08-02 07:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("identities", "0003_historicaluser"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicaluser",
            name="id",
            field=models.BigIntegerField(
                auto_created=True, blank=True, db_index=True, verbose_name="ID"
            ),
        ),
    ]
