# Generated by Django 4.0.6 on 2022-11-04 07:36

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("identities", "0004_alter_historicaluser_id"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalprofile",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical profile",
                "verbose_name_plural": "historical profiles",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaluser",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical user",
                "verbose_name_plural": "historical users",
            },
        ),
        migrations.AlterField(
            model_name="historicalprofile",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaluser",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]
